@@ -13,11 +13,11 @@ use std::fmt;
 use std::fs;
 use std::path::PathBuf;
 
-pub(crate) fn get_manifest(manifest_dir: &Directory) -> Result<Manifest, Error> {
+pub fn get_manifest(manifest_dir: &Directory) -> Result<Manifest, Error> {
     let cargo_toml_path = manifest_dir.join("Cargo.toml");
     let mut manifest = (|| {
         let manifest_str = fs::read_to_string(&cargo_toml_path)?;
-        let manifest: Manifest = toml::from_str(&manifest_str)?;
+        let manifest: Manifest = basic_toml::from_str(&manifest_str)?;
         Ok(manifest)
     })()
     .map_err(|err| Error::GetManifest(cargo_toml_path, Box::new(err)))?;
@@ -32,16 +32,14 @@ pub(crate) fn get_manifest(manifest_dir: &Directory) -> Result<Manifest, Error>
     Ok(manifest)
 }
 
-pub(crate) fn get_workspace_manifest(manifest_dir: &Directory) -> WorkspaceManifest {
+pub fn get_workspace_manifest(manifest_dir: &Directory) -> WorkspaceManifest {
     try_get_workspace_manifest(manifest_dir).unwrap_or_default()
 }
 
-pub(crate) fn try_get_workspace_manifest(
-    manifest_dir: &Directory,
-) -> Result<WorkspaceManifest, Error> {
+pub fn try_get_workspace_manifest(manifest_dir: &Directory) -> Result<WorkspaceManifest, Error> {
     let cargo_toml_path = manifest_dir.join("Cargo.toml");
     let manifest_str = fs::read_to_string(cargo_toml_path)?;
-    let mut manifest: WorkspaceManifest = toml::from_str(&manifest_str)?;
+    let mut manifest: WorkspaceManifest = basic_toml::from_str(&manifest_str)?;
 
     fix_dependencies(&mut manifest.workspace.dependencies, manifest_dir);
     fix_patches(&mut manifest.patch, manifest_dir);
@@ -51,7 +49,7 @@ pub(crate) fn try_get_workspace_manifest(
 }
 
 fn fix_dependencies(dependencies: &mut Map<String, Dependency>, dir: &Directory) {
-    dependencies.remove("trybuild");
+    dependencies.remove("trybuild2");
     for dep in dependencies.values_mut() {
         dep.path = dep.path.as_ref().map(|path| Directory::new(dir.join(path)));
     }
@@ -59,7 +57,7 @@ fn fix_dependencies(dependencies: &mut Map<String, Dependency>, dir: &Directory)
 
 fn fix_patches(patches: &mut Map<String, RegistryPatch>, dir: &Directory) {
     for registry in patches.values_mut() {
-        registry.crates.remove("trybuild");
+        registry.crates.remove("trybuild2");
         for patch in registry.crates.values_mut() {
             patch.path = patch.path.as_ref().map(|path| dir.join(path));
         }
@@ -67,14 +65,14 @@ fn fix_patches(patches: &mut Map<String, RegistryPatch>, dir: &Directory) {
 }
 
 fn fix_replacements(replacements: &mut Map<String, Patch>, dir: &Directory) {
-    replacements.remove("trybuild");
+    replacements.remove("trybuild2");
     for replacement in replacements.values_mut() {
         replacement.path = replacement.path.as_ref().map(|path| dir.join(path));
     }
 }
 
 #[derive(Deserialize, Default, Debug)]
-pub(crate) struct WorkspaceManifest {
+pub struct WorkspaceManifest {
     #[serde(default)]
     pub workspace: WorkspaceWorkspace,
     #[serde(default)]
@@ -84,7 +82,7 @@ pub(crate) struct WorkspaceManifest {
 }
 
 #[derive(Deserialize, Default, Debug)]
-pub(crate) struct WorkspaceWorkspace {
+pub struct WorkspaceWorkspace {
     #[serde(default)]
     pub package: WorkspacePackage,
     #[serde(default)]
@@ -92,14 +90,12 @@ pub(crate) struct WorkspaceWorkspace {
 }
 
 #[derive(Deserialize, Default, Debug)]
-pub(crate) struct WorkspacePackage {
+pub struct WorkspacePackage {
     pub edition: Option<Edition>,
 }
 
 #[derive(Deserialize, Default, Debug)]
-pub(crate) struct Manifest {
-    #[serde(rename = "cargo-features", default)]
-    pub cargo_features: Vec<String>,
+pub struct Manifest {
     #[serde(default)]
     pub package: Package,
     #[serde(default)]
@@ -113,7 +109,7 @@ pub(crate) struct Manifest {
 }
 
 #[derive(Deserialize, Default, Debug)]
-pub(crate) struct Package {
+pub struct Package {
     pub name: String,
     #[serde(default)]
     pub edition: EditionOrInherit,
@@ -121,22 +117,26 @@ pub(crate) struct Package {
 }
 
 #[derive(Debug)]
-pub(crate) enum EditionOrInherit {
+pub enum EditionOrInherit {
     Edition(Edition),
     Inherit,
 }
 
 #[derive(Serialize, Deserialize, Clone, Debug)]
 #[serde(remote = "Self")]
-pub(crate) struct Dependency {
+pub struct Dependency {
     #[serde(skip_serializing_if = "Option::is_none")]
     pub version: Option<String>,
     #[serde(skip_serializing_if = "Option::is_none")]
     pub path: Option<Directory>,
     #[serde(default, skip_serializing_if = "is_false")]
     pub optional: bool,
-    #[serde(rename = "default-features", skip_serializing_if = "Option::is_none")]
-    pub default_features: Option<bool>,
+    #[serde(
+        rename = "default-features",
+        default = "get_true",
+        skip_serializing_if = "is_true"
+    )]
+    pub default_features: bool,
     #[serde(default, skip_serializing_if = "Vec::is_empty")]
     pub features: Vec<String>,
     #[serde(skip_serializing_if = "Option::is_none")]
@@ -154,7 +154,7 @@ pub(crate) struct Dependency {
 }
 
 #[derive(Serialize, Deserialize, Clone, Debug)]
-pub(crate) struct TargetDependencies {
+pub struct TargetDependencies {
     #[serde(default, skip_serializing_if = "Map::is_empty")]
     pub dependencies: Map<String, Dependency>,
     #[serde(
@@ -167,12 +167,12 @@ pub(crate) struct TargetDependencies {
 
 #[derive(Serialize, Deserialize, Clone, Debug)]
 #[serde(transparent)]
-pub(crate) struct RegistryPatch {
+pub struct RegistryPatch {
     pub crates: Map<String, Patch>,
 }
 
 #[derive(Serialize, Deserialize, Clone, Debug)]
-pub(crate) struct Patch {
+pub struct Patch {
     #[serde(skip_serializing_if = "Option::is_none")]
     pub path: Option<PathBuf>,
     #[serde(skip_serializing_if = "Option::is_none")]
@@ -187,6 +187,14 @@ pub(crate) struct Patch {
     pub rest: Map<String, Value>,
 }
 
+fn get_true() -> bool {
+    true
+}
+
+fn is_true(boolean: &bool) -> bool {
+    *boolean
+}
+
 fn is_false(boolean: &bool) -> bool {
     !*boolean
 }
@@ -265,7 +273,7 @@ impl<'de> Deserialize<'de> for Dependency {
                     version: Some(s.to_owned()),
                     path: None,
                     optional: false,
-                    default_features: Some(true),
+                    default_features: true,
                     features: Vec::new(),
                     git: None,
                     branch: None,
