@@ -6,11 +6,10 @@ use self::Normalization::*;
 use crate::directory::Directory;
 use crate::run::PathDependency;
 use std::cmp;
-use std::mem;
 use std::path::Path;
 
 #[derive(Copy, Clone)]
-pub(crate) struct Context<'a> {
+pub struct Context<'a> {
     pub krate: &'a str,
     pub source_dir: &'a Directory,
     pub workspace: &'a Directory,
@@ -58,12 +57,6 @@ normalizations! {
     Unindent,
     AndOthers,
     StripLongTypeNameFiles,
-    UnindentAfterHelp,
-    AndOthersVerbose,
-    UnindentMultilineNote,
-    DependencyVersion,
-    HeadingNote,
-    UnindentSuggestion,
     // New normalization steps are to be inserted here at the end so that any
     // snapshots saved before your normalization change remain passing.
 }
@@ -79,7 +72,7 @@ normalizations! {
 ///
 /// There is one "preferred" variation which is what we print when the stderr
 /// file is absent or not a match.
-pub(crate) fn diagnostics(output: &str, context: Context) -> Variations {
+pub fn diagnostics(output: &str, context: Context) -> Variations {
     let output = output.replace("\r\n", "\n");
 
     let mut result = Variations::default();
@@ -90,7 +83,7 @@ pub(crate) fn diagnostics(output: &str, context: Context) -> Variations {
     result
 }
 
-pub(crate) struct Variations {
+pub struct Variations {
     variations: [String; Normalization::ALL.len()],
 }
 
@@ -113,7 +106,7 @@ impl Variations {
     }
 }
 
-pub(crate) fn trim<S: AsRef<[u8]>>(output: S) -> String {
+pub fn trim<S: AsRef<[u8]>>(output: S) -> String {
     let bytes = output.as_ref();
     let mut normalized = String::from_utf8_lossy(bytes).into_owned();
 
@@ -136,7 +129,6 @@ fn apply(original: &str, normalization: Normalization, context: Context) -> Stri
         normalization,
         context,
         hide_numbers: 0,
-        other_types: None,
     };
     for i in 0..lines.len() {
         if let Some(line) = filter.apply(i) {
@@ -147,7 +139,9 @@ fn apply(original: &str, normalization: Normalization, context: Context) -> Stri
         }
     }
 
-    normalized = unindent(normalized, normalization);
+    if normalization >= Unindent {
+        normalized = unindent(normalized);
+    }
 
     trim(normalized)
 }
@@ -157,7 +151,6 @@ struct Filter<'a> {
     normalization: Normalization,
     context: Context<'a>,
     hide_numbers: usize,
-    other_types: Option<usize>,
 }
 
 impl<'a> Filter<'a> {
@@ -291,7 +284,7 @@ impl<'a> Filter<'a> {
                 } else if line[indent + 4..].starts_with("/rustc/")
                     && line
                         .get(indent + 11..indent + 51)
-                        .is_some_and(is_ascii_lowercase_hex)
+                        .map_or(false, is_ascii_lowercase_hex)
                     && line[indent + 51..].starts_with("/library/")
                 {
                     // --> /rustc/c5c7d2b37780dac1092e75f12ab97dd56c30861e/library/std/src/net/ip.rs:83:1
@@ -309,27 +302,13 @@ impl<'a> Filter<'a> {
                     let hash_end = hash_start + 16;
                     if line
                         .get(hash_start..hash_end)
-                        .is_some_and(is_ascii_lowercase_hex)
+                        .map_or(false, is_ascii_lowercase_hex)
                         && line[hash_end..].starts_with('/')
                     {
                         // --> /home/.cargo/registry/src/github.com-1ecc6299db9ec823/serde_json-1.0.64/src/de.rs:2584:8
                         // --> $CARGO/serde_json-1.0.64/src/de.rs:2584:8
                         line.replace_range(indent + 4..hash_end, "$CARGO");
                         other_crate = true;
-                        if self.normalization >= DependencyVersion {
-                            let rest = &line[indent + 11..];
-                            let end_of_version = rest.find('/');
-                            if let Some(end_of_crate_name) = end_of_version
-                                .and_then(|end| rest[..end].find('.'))
-                                .and_then(|end| rest[..end].rfind('-'))
-                            {
-                                line.replace_range(
-                                    indent + end_of_crate_name + 12
-                                        ..indent + end_of_version.unwrap() + 11,
-                                    "$VERSION",
-                                );
-                            }
-                        }
                     }
                 }
             }
@@ -358,12 +337,6 @@ impl<'a> Filter<'a> {
             return None;
         }
 
-        if trim_start.starts_with("= note: this compiler was built on 2")
-            && trim_start.ends_with("; consider upgrading it if it is out of date")
-        {
-            return None;
-        }
-
         if self.normalization >= StripCouldNotCompile {
             if line.starts_with("error: Could not compile `") {
                 return None;
@@ -423,39 +396,11 @@ impl<'a> Filter<'a> {
             let trimmed_line = trimmed_line
                 .strip_prefix("= note: ")
                 .unwrap_or(trimmed_line);
-            if trimmed_line.starts_with("the full type name has been written to")
-                || trimmed_line.starts_with("the full name for the type has been written to")
-            {
+            if trimmed_line.starts_with("the full type name has been written to") {
                 return None;
             }
         }
 
-        if self.normalization >= AndOthersVerbose {
-            let trim_start = line.trim_start();
-            if trim_start.starts_with("= help: the following types implement trait ")
-                || trim_start.starts_with("= help: the following other types implement trait ")
-            {
-                self.other_types = Some(0);
-            } else if let Some(count_other_types) = &mut self.other_types {
-                if indent >= 12 && trim_start != "and $N others" {
-                    *count_other_types += 1;
-                    if *count_other_types == 9 {
-                        if let Some(next) = self.all_lines.get(index + 1) {
-                            let next_trim_start = next.trim_start();
-                            let next_indent = next.len() - next_trim_start.len();
-                            if indent == next_indent {
-                                line.replace_range(indent - 2.., "and $N others");
-                            }
-                        }
-                    } else if *count_other_types > 9 {
-                        return None;
-                    }
-                } else {
-                    self.other_types = None;
-                }
-            }
-        }
-
         line = line.replace(self.context.krate, "$CRATE");
         line = replace_case_insensitive(&line, &self.context.source_dir.to_string_lossy(), "$DIR/");
         line = replace_case_insensitive(
@@ -474,10 +419,7 @@ fn is_ascii_lowercase_hex(s: &str) -> bool {
 
 // "10 | T: Send,"  ->  "   | T: Send,"
 fn hide_leading_numbers(line: &mut String) {
-    let n = line
-        .bytes()
-        .take_while(|b: &u8| *b == b' ' || b.is_ascii_digit())
-        .count();
+    let n = line.bytes().take_while(u8::is_ascii_digit).count();
     for i in 0..n {
         line.replace_range(i..i + 1, " ");
     }
@@ -552,11 +494,7 @@ enum IndentedLineKind {
     Other(usize),
 }
 
-fn unindent(diag: String, normalization: Normalization) -> String {
-    if normalization < Unindent {
-        return diag;
-    }
-
+fn unindent(diag: String) -> String {
     let mut normalized = String::new();
     let mut lines = diag.lines();
 
@@ -564,25 +502,22 @@ fn unindent(diag: String, normalization: Normalization) -> String {
         normalized.push_str(line);
         normalized.push('\n');
 
-        if indented_line_kind(line, true, &mut false, normalization) != IndentedLineKind::Heading {
+        if indented_line_kind(line) != IndentedLineKind::Heading {
             continue;
         }
 
         let mut ahead = lines.clone();
-        let Some(next_line) = ahead.next() else {
-            continue;
+        let next_line = match ahead.next() {
+            Some(line) => line,
+            None => continue,
         };
 
-        if let IndentedLineKind::Code(indent) =
-            indented_line_kind(next_line, false, &mut false, normalization)
-        {
+        if let IndentedLineKind::Code(indent) = indented_line_kind(next_line) {
             if next_line[indent + 1..].starts_with("--> ") {
                 let mut lines_in_block = 1;
                 let mut least_indent = indent;
-                let mut previous_line_is_note = false;
                 while let Some(line) = ahead.next() {
-                    match indented_line_kind(line, false, &mut previous_line_is_note, normalization)
-                    {
+                    match indented_line_kind(line) {
                         IndentedLineKind::Heading => break,
                         IndentedLineKind::Code(indent) => {
                             lines_in_block += 1;
@@ -598,11 +533,10 @@ fn unindent(diag: String, normalization: Normalization) -> String {
                         }
                     }
                 }
-                previous_line_is_note = false;
                 for _ in 0..lines_in_block {
                     let line = lines.next().unwrap();
                     if let IndentedLineKind::Code(_) | IndentedLineKind::Other(_) =
-                        indented_line_kind(line, false, &mut previous_line_is_note, normalization)
+                        indented_line_kind(line)
                     {
                         let space = line.find(' ').unwrap();
                         normalized.push_str(&line[..space]);
@@ -619,14 +553,7 @@ fn unindent(diag: String, normalization: Normalization) -> String {
     normalized
 }
 
-fn indented_line_kind(
-    line: &str,
-    first_line_in_block: bool,
-    previous_line_is_note: &mut bool,
-    normalization: Normalization,
-) -> IndentedLineKind {
-    let previous_line_was_note = mem::replace(previous_line_is_note, false);
-
+fn indented_line_kind(line: &str) -> IndentedLineKind {
     if let Some(heading_len) = if line.starts_with("error") {
         Some("error".len())
     } else if line.starts_with("warning") {
@@ -639,18 +566,7 @@ fn indented_line_kind(
         }
     }
 
-    if first_line_in_block && normalization >= HeadingNote && line.starts_with("note: ") {
-        return IndentedLineKind::Heading;
-    }
-
-    if line.starts_with("note:")
-        || line == "..."
-        || normalization >= UnindentAfterHelp && line.starts_with("help:")
-        || normalization >= UnindentMultilineNote
-            && previous_line_was_note
-            && line.starts_with("      ")
-    {
-        *previous_line_is_note = true;
+    if line.starts_with("note:") || line == "..." {
         return IndentedLineKind::Note;
     }
 
@@ -660,24 +576,12 @@ fn indented_line_kind(
         return IndentedLineKind::Code(spaces);
     }
 
-    let mut spaces = line.bytes().take_while(is_space).count();
-    let digits = line[spaces..]
-        .bytes()
-        .take_while(u8::is_ascii_digit)
-        .count();
-    spaces += line[spaces + digits..].bytes().take_while(is_space).count();
+    let digits = line.bytes().take_while(u8::is_ascii_digit).count();
+    let spaces = line[digits..].bytes().take_while(|b| *b == b' ').count();
     let rest = &line[digits + spaces..];
     if spaces > 0
         && (rest == "|"
             || rest.starts_with("| ")
-            || normalization >= UnindentSuggestion
-                && digits > 0
-                && (rest == "~"
-                    || rest.starts_with("~ ")
-                    || rest == "+"
-                    || rest.starts_with("+ ")
-                    || rest == "-"
-                    || rest.starts_with("- "))
             || digits == 0
                 && (rest.starts_with("--> ") || rest.starts_with("::: ") || rest.starts_with("= ")))
     {
