@@ -5,21 +5,22 @@ use crate::env::Update;
 use crate::error::{Error, Result};
 use crate::expand::{expand_globs, ExpandedTest};
 use crate::flock::Lock;
-use crate::manifest::{Bin, Manifest, Name, Package, Workspace};
+use crate::manifest::{Bin, Build, Config, Manifest, Name, Package, Workspace};
 use crate::message::{self, Fail, Warn};
 use crate::normalize::{self, Context, Variations};
-use crate::{features, Expected, Runner, Test};
+use crate::{features, rustflags, Expected, InlineTest, Runner, Test, TestKind};
 use serde_derive::Deserialize;
 use std::collections::{BTreeMap as Map, BTreeSet as Set};
 use std::env;
 use std::ffi::{OsStr, OsString};
-use std::fs::{self, File};
+use std::fs::{self, File, OpenOptions};
+use std::io::Write;
 use std::mem;
 use std::path::{Path, PathBuf};
 use std::str;
 
 #[derive(Debug)]
-pub(crate) struct Project {
+pub struct Project {
     pub dir: Directory,
     source_dir: Directory,
     pub target_dir: Directory,
@@ -35,7 +36,7 @@ pub(crate) struct Project {
 }
 
 #[derive(Debug)]
-pub(crate) struct PathDependency {
+pub struct PathDependency {
     pub name: String,
     pub normalized_path: Directory,
 }
@@ -46,13 +47,13 @@ struct Report {
 }
 
 impl Runner {
-    pub(crate) fn run(&mut self) {
+    pub fn run(&mut self) {
         let mut tests = expand_globs(&self.tests);
         filter(&mut tests);
 
         let (project, _lock) = (|| {
             let mut project = self.prepare(&tests)?;
-            let lock = Lock::acquire(path!(project.dir / ".lock"))?;
+            let lock = Lock::acquire(path!(project.dir / ".lock"));
             self.write(&mut project)?;
             Ok((project, lock))
         })()
@@ -97,10 +98,10 @@ impl Runner {
 
         print!("\n\n");
 
-        if report.failures > 0 && project.name != "trybuild-tests" {
+        if report.failures > 0 && project.name != "trybuild2-tests" {
             panic!("{} of {} tests failed", report.failures, len);
         }
-        if report.created_wip > 0 && project.name != "trybuild-tests" {
+        if report.created_wip > 0 && project.name != "trybuild2-tests" {
             panic!(
                 "successfully created new stderr files for {} test cases",
                 report.created_wip,
@@ -119,8 +120,10 @@ impl Runner {
         let mut has_compile_fail = false;
         for e in tests {
             match e.test.expected {
-                Expected::Pass => has_pass = true,
-                Expected::CompileFail => has_compile_fail = true,
+                Expected::Pass | Expected::PassSubString(_) => has_pass = true,
+                Expected::CompileFail | Expected::CompileFailSubString(_) => {
+                    has_compile_fail = true;
+                }
             }
         }
 
@@ -154,6 +157,7 @@ impl Runner {
         let manifest = self.make_manifest(
             &workspace,
             &project_name,
+            &project_dir,
             &source_dir,
             &packages,
             tests,
@@ -181,11 +185,17 @@ impl Runner {
     }
 
     fn write(&self, project: &mut Project) -> Result<()> {
-        let manifest_toml = toml::to_string(&project.manifest)?;
+        let manifest_toml = basic_toml::to_string(&project.manifest)?;
+
+        let config = self.make_config();
+        let config_toml = basic_toml::to_string(&config)?;
+
+        fs::create_dir_all(path!(project.dir / ".cargo"))?;
+        fs::write(path!(project.dir / ".cargo" / "config.toml"), config_toml)?;
         fs::write(path!(project.dir / "Cargo.toml"), manifest_toml)?;
 
         let main_rs = b"\
-            #![allow(unused_crate_dependencies, missing_docs)]\n\
+            #![allow(unknown_lints, unused_crate_dependencies, missing_docs)]\n\
             fn main() {}\n\
         ";
         fs::write(path!(project.dir / "main.rs"), &main_rs[..])?;
@@ -195,10 +205,12 @@ impl Runner {
         Ok(())
     }
 
+    #[allow(clippy::too_many_arguments)]
     fn make_manifest(
         &self,
         workspace: &Directory,
         project_name: &str,
+        project_dir: &Directory,
         source_dir: &Directory,
         packages: &[PackageMetadata],
         tests: &[ExpandedTest],
@@ -237,7 +249,7 @@ impl Runner {
                     version: None,
                     path: Some(source_dir.clone()),
                     optional: false,
-                    default_features: Some(false),
+                    default_features: false,
                     features: Vec::new(),
                     git: None,
                     branch: None,
@@ -258,8 +270,9 @@ impl Runner {
         let mut features = source_manifest.features;
         for (feature, enables) in &mut features {
             enables.retain(|en| {
-                let Some(dep_name) = en.strip_prefix("dep:") else {
-                    return false;
+                let dep_name = match en.strip_prefix("dep:") {
+                    Some(dep_name) => dep_name,
+                    None => return false,
                 };
                 if let Some(Dependency { optional: true, .. }) = dependencies.get(dep_name) {
                     return true;
@@ -279,7 +292,6 @@ impl Runner {
         }
 
         let mut manifest = Manifest {
-            cargo_features: source_manifest.cargo_features,
             package: Package {
                 name: project_name.to_owned(),
                 version: "0.0.0".to_owned(),
@@ -307,9 +319,13 @@ impl Runner {
 
         for expanded in tests {
             if expanded.error.is_none() {
+                let path = match expanded.test.inner {
+                    TestKind::File => source_dir.join(&expanded.test.path),
+                    TestKind::Inline(ref t) => project_dir.join(&format!("{}.rs", t.name)),
+                };
                 manifest.bins.push(Bin {
                     name: expanded.name.clone(),
-                    path: source_dir.join(&expanded.test.path),
+                    path: source_dir.join(&path),
                 });
             }
         }
@@ -317,15 +333,36 @@ impl Runner {
         Ok(manifest)
     }
 
-    fn run_all(&self, project: &Project, tests: Vec<ExpandedTest>) -> Result<Report> {
+    fn make_config(&self) -> Config {
+        Config {
+            build: Build {
+                rustflags: rustflags::make_vec(),
+            },
+        }
+    }
+
+    fn run_all(&self, project: &Project, mut tests: Vec<ExpandedTest>) -> Result<Report> {
         let mut report = Report {
             failures: 0,
             created_wip: 0,
         };
 
         let mut path_map = Map::new();
-        for t in &tests {
-            let src_path = project.source_dir.join(&t.test.path);
+        for t in &mut tests {
+            let src_path = match t.test.inner {
+                TestKind::File => {
+                    if t.error.is_none() {
+                        t.error = check_exists(&t.test.path).err();
+                    }
+                    project.source_dir.join(&t.test.path)
+                }
+                TestKind::Inline(ref inl) => {
+                    if t.error.is_none() {
+                        t.error = create_inline_test(inl, project).err();
+                    }
+                    project.dir.join(&format!("{}.rs", inl.name))
+                }
+            };
             path_map.insert(src_path, (&t.name, &t.test));
         }
 
@@ -337,17 +374,28 @@ impl Runner {
             let show_expected = false;
             message::begin_test(&t.test, show_expected);
 
+            let mut src_path = None;
+            let mut stderr_path = None;
             if t.error.is_none() {
-                t.error = check_exists(&t.test.path).err();
+                match t.test.inner {
+                    TestKind::File => {
+                        src_path = Some(project.source_dir.join(&t.test.path));
+                    }
+                    TestKind::Inline(ref inl) => {
+                        src_path = Some(project.dir.join(&format!("{}.rs", inl.name)));
+                        stderr_path = inl.stderr_path.clone();
+                    }
+                }
             }
 
             if t.error.is_none() {
-                let src_path = project.source_dir.join(&t.test.path);
-                let this_test = parsed.stderrs.get(&src_path).unwrap_or(&fallback);
-                match t.test.check(project, &t.name, this_test, "") {
-                    Ok(Outcome::Passed) => {}
-                    Ok(Outcome::CreatedWip) => report.created_wip += 1,
-                    Err(error) => t.error = Some(error),
+                if let Some(src_path) = src_path {
+                    let this_test = parsed.stderrs.get(&src_path).unwrap_or(&fallback);
+                    match t.test.check(project, &t.name, this_test, "", stderr_path) {
+                        Ok(Outcome::Passed) => {}
+                        Ok(Outcome::CreatedWip) => report.created_wip += 1,
+                        Err(error) => t.error = Some(error),
+                    }
                 }
             }
 
@@ -370,17 +418,29 @@ impl Test {
     fn run(&self, project: &Project, name: &Name) -> Result<Outcome> {
         let show_expected = project.has_pass && project.has_compile_fail;
         message::begin_test(self, show_expected);
-        check_exists(&self.path)?;
+
+        let src_path;
+        let stderr_path;
+        match self.inner {
+            TestKind::File => {
+                check_exists(&self.path)?;
+                src_path = project.source_dir.join(&self.path);
+                stderr_path = None;
+            }
+            TestKind::Inline(ref t) => {
+                create_inline_test(t, project)?;
+                src_path = project.dir.join(format!("{}.rs", t.name));
+                stderr_path = t.stderr_path.clone();
+            }
+        }
 
         let mut path_map = Map::new();
-        let src_path = project.source_dir.join(&self.path);
         path_map.insert(src_path.clone(), (name, self));
-
         let output = cargo::build_test(project, name)?;
         let parsed = parse_cargo_json(project, &output.stdout, &path_map);
         let fallback = Stderr::default();
         let this_test = parsed.stderrs.get(&src_path).unwrap_or(&fallback);
-        self.check(project, name, this_test, &parsed.stdout)
+        self.check(project, name, this_test, &parsed.stdout, stderr_path)
     }
 
     fn check(
@@ -389,20 +449,44 @@ impl Test {
         name: &Name,
         result: &Stderr,
         build_stdout: &str,
+        stderr_path: Option<PathBuf>,
     ) -> Result<Outcome> {
-        let check = match self.expected {
-            Expected::Pass => Test::check_pass,
-            Expected::CompileFail => Test::check_compile_fail,
-        };
-
-        check(
-            self,
-            project,
-            name,
-            result.success,
-            build_stdout,
-            &result.stderr,
-        )
+        match self.expected {
+            Expected::Pass => Test::check_pass(
+                self,
+                project,
+                name,
+                result.success,
+                build_stdout,
+                &result.stderr,
+            ),
+            Expected::PassSubString(ref s) => Test::check_pass_sub_str(
+                self,
+                project,
+                name,
+                result.success,
+                build_stdout,
+                &result.stderr,
+                s,
+            ),
+            Expected::CompileFail => Test::check_compile_fail(
+                self,
+                project,
+                name,
+                result.success,
+                build_stdout,
+                &result.stderr,
+                stderr_path,
+            ),
+            Expected::CompileFailSubString(ref s) => Test::check_compile_fail_sub_str(
+                self,
+                name,
+                result.success,
+                build_stdout,
+                &result.stderr,
+                s,
+            ),
+        }
     }
 
     fn check_pass(
@@ -429,6 +513,34 @@ impl Test {
         }
     }
 
+    fn check_pass_sub_str(
+        &self,
+        project: &Project,
+        name: &Name,
+        success: bool,
+        build_stdout: &str,
+        variations: &Variations,
+        expected: &str,
+    ) -> Result<Outcome> {
+        let preferred = variations.preferred();
+        if !success {
+            message::failed_to_build(preferred);
+            return Err(Error::CargoFail);
+        }
+
+        let mut output = cargo::run_test(project, name)?;
+        output.stdout.splice(..0, build_stdout.bytes());
+        if !output.status.success() {
+            return Err(Error::RunFailed);
+        }
+        if variations.any(|stderr| stderr.contains(expected)) {
+            message::ok();
+            return Ok(Outcome::Passed);
+        }
+        message::mismatch(expected, preferred, "");
+        Err(Error::Mismatch)
+    }
+
     fn check_compile_fail(
         &self,
         project: &Project,
@@ -436,6 +548,7 @@ impl Test {
         success: bool,
         build_stdout: &str,
         variations: &Variations,
+        stderr_path: Option<PathBuf>,
     ) -> Result<Outcome> {
         let preferred = variations.preferred();
 
@@ -446,7 +559,7 @@ impl Test {
             return Err(Error::ShouldNotHaveCompiled);
         }
 
-        let stderr_path = self.path.with_extension("stderr");
+        let stderr_path = stderr_path.unwrap_or_else(|| self.path.with_extension("stderr"));
 
         if !stderr_path.exists() {
             let outcome = match project.update {
@@ -484,7 +597,7 @@ impl Test {
 
         match project.update {
             Update::Wip => {
-                message::mismatch(&expected, preferred);
+                message::mismatch(&expected, preferred, "");
                 Err(Error::Mismatch)
             }
             Update::Overwrite => {
@@ -494,6 +607,32 @@ impl Test {
             }
         }
     }
+
+    fn check_compile_fail_sub_str(
+        &self,
+        _name: &Name,
+        success: bool,
+        build_stdout: &str,
+        variations: &Variations,
+        expected: &str,
+    ) -> Result<Outcome> {
+        let preferred = variations.preferred();
+
+        if success {
+            message::should_not_have_compiled();
+            message::fail_output(Fail, build_stdout);
+            message::warnings(preferred);
+            return Err(Error::ShouldNotHaveCompiled);
+        }
+
+        if variations.any(|stderr| stderr.contains(expected)) {
+            message::ok();
+            return Ok(Outcome::Passed);
+        }
+
+        message::mismatch(expected, preferred, "");
+        Err(Error::Mismatch)
+    }
 }
 
 fn check_exists(path: &Path) -> Result<()> {
@@ -506,6 +645,17 @@ fn check_exists(path: &Path) -> Result<()> {
     }
 }
 
+fn create_inline_test(test: &InlineTest, project: &Project) -> Result<()> {
+    let path = path!(project.dir / format!("{}.rs", test.name));
+    let mut file = OpenOptions::new()
+        .write(true)
+        .truncate(true)
+        .create(true)
+        .open(path)
+        .map_err(Error::FileCreation)?;
+    write!(file, "{}", test.code).map_err(Error::FileCreation)
+}
+
 impl ExpandedTest {
     fn run(self, project: &Project) -> Result<Outcome> {
         match self.error {
@@ -519,13 +669,13 @@ impl ExpandedTest {
     }
 }
 
-// Filter which test cases are run by trybuild.
+// Filter which test cases are run by trybuild2.
 //
-//     $ cargo test -- ui trybuild=tuple_structs.rs
+//     $ cargo test -- ui trybuild2=tuple_structs.rs
 //
-// The first argument after `--` must be the trybuild test name i.e. the name of
-// the function that has the #[test] attribute and calls trybuild. That's to get
-// Cargo to run the test at all. The next argument starting with `trybuild=`
+// The first argument after `--` must be the trybuild2 test name i.e. the name of
+// the function that has the #[test] attribute and calls trybuild2. That's to get
+// Cargo to run the test at all. The next argument starting with `trybuild2=`
 // provides a filename filter. Only test cases whose filename contains the
 // filter string will be run.
 #[allow(clippy::needless_collect)] // false positive https://github.com/rust-lang/rust-clippy/issues/5991
@@ -533,7 +683,7 @@ fn filter(tests: &mut Vec<ExpandedTest>) {
     let filters = env::args_os()
         .flat_map(OsString::into_string)
         .filter_map(|mut arg| {
-            const PREFIX: &str = "trybuild=";
+            const PREFIX: &str = "trybuild2=";
             if arg.starts_with(PREFIX) && arg != PREFIX {
                 Some(arg.split_off(PREFIX.len()))
             } else {
@@ -607,8 +757,9 @@ fn parse_cargo_json(
     let mut remaining = &*String::from_utf8_lossy(stdout);
     let mut seen = Set::new();
     while !remaining.is_empty() {
-        let Some(begin) = remaining.find("{\"reason\":") else {
-            break;
+        let begin = match remaining.find("{\"reason\":") {
+            Some(begin) => begin,
+            None => break,
         };
         let (nonmessage, rest) = remaining.split_at(begin);
         nonmessage_stdout.push_str(nonmessage);
@@ -627,8 +778,9 @@ fn parse_cargo_json(
         }
         if let Ok(de) = serde_json::from_str::<CargoMessage>(message) {
             if de.message.level != "failure-note" {
-                let Some((name, test)) = path_map.get(&de.target.src_path) else {
-                    continue;
+                let (name, test) = match path_map.get(&de.target.src_path) {
+                    Some(test) => test,
+                    None => continue,
                 };
                 let entry = map
                     .entry(de.target.src_path)
